﻿/*
    StudiVZExtractor - extracts information from your StudiVZ account
    Copyright (C) 2009 Andreas 'BlackHC' Kirsch

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Web;
using System.IO;
using System.Diagnostics;
using System.Text.RegularExpressions;

namespace StudiVZExtractor {
    class Program {
        // we need a cookie store for StudiVZ's login cookies
        // from http://stackoverflow.com/questions/571964/automatic-cookie-handling-c-net-httpwebrequesthttpwebresponse
        static CookieContainer cookieJar = new CookieContainer();

        // get the page at url using active cookies
        static string GetPage(string url) {
            HttpWebRequest req = (HttpWebRequest) WebRequest.Create(url);

            req.CookieContainer = cookieJar;

            // Set values for the request back
            req.Method = "GET";

            // Do the request to get the response
            StreamReader stIn = new StreamReader(req.GetResponse().GetResponseStream());
            string strResponse = stIn.ReadToEnd();
            stIn.Close();

            return strResponse;
        }

        // some hidden values needed for login
        static string iv, formkey;

        static void InitIVAndFormkey() {
            string page = GetPage("http://www.studivz.net/");

            Match ivResult = Regex.Match(page, "name=\"iv\"\\s*value=\"([^\"]*)\"");
            iv = ivResult.Groups[1].Value;

            Match formkeyResult = Regex.Match(page, "name=\"formkey\"\\s*value=\"([^\"]*)\"");
            formkey = formkeyResult.Groups[1].Value;
        }

        static void LogIntoStudiVZ(string account, string password) {
            // from http://www.csharpfriends.com/Forums/ShowPost.aspx?PostID=32254
            string strNewValue;
            string strResponse;

            // Create the request obj
            HttpWebRequest request = (HttpWebRequest) WebRequest.Create("https://secure.studivz.net/Login");

            request.CookieContainer = cookieJar;
           
            // Set values for the request back
            request.Method = "POST";
            request.ContentType = "application/x-www-form-urlencoded";
            // stupid warning if jsEnabled=false (thanks HTML Visualizer :))
            strNewValue = "email=" + account + "&password=" + password + "&iv=" + iv + "&formkey=" + formkey + "&jsEnabled=true&login=Einloggen";
            request.ContentLength = strNewValue.Length;

            // Write the request
            StreamWriter stOut = new StreamWriter(request.GetRequestStream(), System.Text.Encoding.ASCII);
            stOut.Write(strNewValue);
            stOut.Close();

            HttpWebResponse response = (HttpWebResponse) request.GetResponse();
            
            // Do the request to get the response
            StreamReader stIn = new StreamReader(response.GetResponseStream());
            strResponse = stIn.ReadToEnd();
            stIn.Close();

            // from MSDN: HttpWebRequest..::.CookieContainer Property 
            /*
            // Print the properties of each cookie.
            foreach (Cookie cook in response.Cookies) {
                Console.WriteLine("Cookie:");
                Console.WriteLine("{0} = {1}", cook.Name, cook.Value);
                Console.WriteLine("Domain: {0}", cook.Domain);
                Console.WriteLine("Path: {0}", cook.Path);
                Console.WriteLine("Port: {0}", cook.Port);
                Console.WriteLine("Secure: {0}", cook.Secure);

                Console.WriteLine("When issued: {0}", cook.TimeStamp);
                Console.WriteLine("Expires: {0} (expired? {1})",
                    cook.Expires, cook.Expired);
                Console.WriteLine("Don't save: {0}", cook.Discard);
                Console.WriteLine("Comment: {0}", cook.Comment);
                Console.WriteLine("Uri for comments: {0}", cook.CommentUri);
                Console.WriteLine("Version: RFC {0}", cook.Version == 1 ? "2109" : "2965");

                // Show the string representation of the cookie.
                Console.WriteLine("String: {0}", cook.ToString());
            }*/
        }

        struct Friend {
            public string friendListEntry;

            public string profileURL;
            public string profileID;

            public string name;
        }

        static string friendsURL;
        static List<Friend> friendsList = new List<Friend>();

        static void InitFriendsURL() {
            string page = GetPage("http://www.studivz.net/");

            Match friendURLMatch = Regex.Match(page, "href=\"([^\"]*)\" rel=\"nofollow\" title=\"Meine Freunde\"");
            string url = friendURLMatch.Groups[1].Value;
            friendsURL = url.Replace("/tid/103", "/p/");
        }

        // get the indexth "Meine Freunde" page 
        static string GetFriendsPage(int pageIndex) {
            return GetPage( "http://www.studivz.net" + friendsURL + pageIndex);
        }

        static void RetrieveFriends() {
            // thanks to http://derekslager.com/blog/posts/2007/09/a-better-dotnet-regular-expression-tester.ashx
            Regex firendEntryRegex = new Regex(@"\<tr(?:.|\n)*?\/tr\>", RegexOptions.Multiline);
            Regex friendInfoRegex = new Regex( "\\<a href=\"(/Profile/(?<pindex>[^\"]*))\"\\s*id=\"name_\\k<pindex>\"\\s*>([^\\<]*)\\<" );
           
            int pageIndex = 1;
            while( true ) {
                string page = GetFriendsPage(pageIndex++);

                MatchCollection friendEntries = firendEntryRegex.Matches(page);
                foreach(Match friendEntry in friendEntries) {
                    Friend aFriend = new Friend();
                    aFriend.friendListEntry = friendEntry.Groups[0].Value;

                    Match friendInfo = friendInfoRegex.Match(aFriend.friendListEntry);
                    aFriend.profileURL = friendInfo.Groups[1].Value;
                    aFriend.name = friendInfo.Groups[2].Value;
                    aFriend.profileID = friendInfo.Groups[3].Value;

                    friendsList.Add(aFriend);
                }
                if (friendEntries.Count == 0)
                    break;
            } 
        }

        static void Main(string[] args){
            Console.WriteLine(
             "StudiVZExtractor  Copyright (C) 2009 Andreas 'BlackHC' Kirsch\n" +
             "This program comes with ABSOLUTELY NO WARRANTY.\n" +
             "This is free software, and you are welcome to redistribute it\n" +
             "under the conditions of the GPL.\n"
            );

            if (args.Length != 2) {
                Console.WriteLine("<email> <password>");
                return;
            }
            InitIVAndFormkey();

            LogIntoStudiVZ(args[0], args[1]);

            InitFriendsURL();

            RetrieveFriends();

            Console.WriteLine("You have {0} friends", friendsList.Count);
            foreach (Friend myFriend in friendsList) {
                Console.WriteLine("{0}\t\t\tProfile ID: {1} ( {2} )", myFriend.name, myFriend.profileID, myFriend.profileURL );
            }
        }
    }
}
